/*
 * ShowOff Widget 1.21
 * 2005 Joakim Nygrd, all rights reserver
 * Portions of this software is 2005 Oliver Sprenger
 *
 * This Widget is provided as is with no warranty and
 * no guarantees.
 * You freely distribute this software without modifications.
 * If you modify this Widget, you must give the author credit.
 * If you read this, be inspired by the rest of this file :)
 */

// defines the different display modes.
var currentDisplay = 0;

// this holds the record uptime
var recordKey = 0;
var timer = null;

function getLocalizedString (key)
{
	try {
		var ret = localizedStrings[key];
		if (ret === undefined)
			ret = key;
		return ret;
	} catch (ex) {}

	return key;
}

function setup() {
	if (window.widget)
	{
	    widget.onshow = onshow;
	    widget.onhide = onhide;
	
		readPrefs();
	}
	createGenericButton(document.getElementById('done'), getLocalizedString('Back'), hidePrefs);
	onshow();
}

// toggle between the display modes
function toggleDisplay() {
	// we have 3 settings, loop through them
	currentDisplay = ++currentDisplay % 3;
	onshow();
}

// open Safari and go to the url given
function goToUrl(url)
{
	if (window.widget)
	{
		widget.openURL(url);
	}
}

function onshow() {
	readPrefs();

	if (timer == null) {
		updateDisplay();
		timer = setInterval("updateDisplay();", 2000);
	}
}

// Display the info requested as set by the currentDisplay variable
// Called every 2 seconds
function updateDisplay() {
	
	if(currentDisplay==0) {
		document.getElementById("uptime").innerText = readable(fetchUptime());
		document.getElementById("info").innerText = getLocalizedString("Uptime");
	} 
	else if(currentDisplay==1) {
		document.getElementById("uptime").innerText = readable(fetchRecord());
		document.getElementById("info").innerText = getLocalizedString("Record");
	}
	else {
		document.getElementById("uptime").innerText = fetchAverages();
		document.getElementById("info").innerText = getLocalizedString("Load");	
	}
}

function onhide() {
	var currentUp = fetchUptime();
	if(currentUp > recordKey) {
		widget.setPreferenceForKey(currentUp, "record");
	}
	if (timer != null) {
        clearInterval(timer);
        timer = null;
    }
}

// method contributed by Oliver Sprenger, dev@sprenger-mail.de
function fetchAverages() {
    outString = widget.system("/usr/bin/uptime", null).outputString;
    return outString.substring(outString.indexOf('averages')+10);
}

function fetchUptime() {
	//debug value;
	//outString = "18:44  up 21 mins, 8 users, load averages: 1.61 1.76 1.67";
	outString = widget.system("/usr/bin/uptime", null).outputString;
	var upString = outString.substring(0,outString.indexOf("load")-2);
	upString = upString.substring(upString.indexOf("up")+3,upString.lastIndexOf(","));
	return getMinutes(upString.split(","));
}

function fetchRecord() {
	var currentUp = fetchUptime();
	
	// is old record still valid?
	if(recordKey != 0 && recordKey > currentUp) {
	    return recordKey;
	}
	// else new record
	else {
		return currentUp;
	}
}

// method contributed by Oliver Sprenger, dev@sprenger-mail.de
function readable(string) {
		var tmp = string % 1440;
		var days = (string-(string % 1440))/1440; 
		var hours = (tmp-(tmp % 60))/60; 
		var minute = tmp % 60;
		var verboseString = "";
		
		if(days > 0) {
			if(days == 1)
				verboseString += days+ getLocalizedString(' day');
			else
				verboseString += days+ getLocalizedString(' days');
		}
		
		if(hours > 0) {
		    if (verboseString != "")
		       verboseString += ", ";
		       
			if(hours < 10)
				verboseString += "0";
			verboseString += hours+":";
			
			if(minute < 10)
			    verboseString += "0";
	    	verboseString += minute+"";
		}
      else if (minute > 0)
		{
		    if (verboseString != "")
		       verboseString += ", ";
		
		    if(minute < 10)
			    verboseString += "0";
		    verboseString += minute;
		    
		    if (minute == 1)
			   verboseString += getLocalizedString(' minute');
			else
			   verboseString += getLocalizedString(' minutes');
        }
		return verboseString;
}

function getMinutes(string) {
	var currentUp = 0;
	for(i=string.length-1;i>=0;i--) {
		// minutes and hours
		if(i==string.length-1) {
			var timePart = string[i].split(":");
			// we have both hours and minutes
			if(timePart.length > 1) {
				currentUp = Number(timePart[1]);
				currentUp += Number(timePart[0]*60);
			}
			// either less than one hour or a whole amount of hours
			else {
				timePart = string[i].split(" ");
				// remove extra array element
				if(timePart[0]=="") {
					timePart = timePart.slice(1);
				}
				if(timePart[1]=="hrs" || timePart[1]=="hours" || timePart[1]=="hour") {
					currentUp = Number(timePart[0]*60);
				} else if(timePart[1]=="min" || timePart[1]=="mins") {
					currentUp = Number(timePart[0]);
				}
			}
		}
		// we have days
		else if(i==string.length-2) {
			timePart = string[i].split(" ");
			currentUp += Number(timePart[0]*24*60);
		}
		// reaching weeks
		else if(i==string.length-3) {
			timePart = string[i].split(" ");
			currentUp += Number(timePart[0]*7*24*60);
		}
	}
	return currentUp;
}

function readPrefs() {
	if(window.widget) {
		recordKey = widget.preferenceForKey("record");
		if (isNaN(recordKey))
		{
			recordKey = 0;
		}
	}
}

function showPrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget)
		widget.prepareForTransition("ToBack");
       
	front.style.display="none";
	back.style.display="block";

	if (window.widget)
		setTimeout ('widget.performTransition();', 0);  
}

function hidePrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget)
		widget.prepareForTransition("ToFront");
      
	back.style.display="none";
	front.style.display="block";

	if (window.widget)
		setTimeout ('widget.performTransition();', 0);
}

var flipShown = false;
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null};

function mousemove (event)
{
	if (!flipShown)
	{
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
 
		var starttime = (new Date).getTime() - 13;
 
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}
function mouseexit (event)
{
	if (flipShown)
	{
		// fade in the info button
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}

		var starttime = (new Date).getTime() - 13;

		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}
function animate()
{
	var T;
	var ease;
	var time = (new Date).getTime();
   

	T = limit_3(time-animation.starttime, 0, animation.duration);

	if (T >= animation.duration)
	{
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else
	{
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}

	animation.firstElement.style.opacity = animation.now;
}
function limit_3 (a, b, c)
{
	return a < b ? b : (a > c ? c : a);
}
function computeNextFloat (from, to, ease)
{
	return from + (to - from) * ease;
}

// these functions are called when the info button itself receives onmouseover and onmouseout events

function enterflip(event)
{
	document.getElementById('fliprollie').style.display = 'block';
}

function exitflip(event)
{
	document.getElementById('fliprollie').style.display = 'none';
}